@@ -24,7 +24,7 @@ extern "C" {
 #include <stdbool.h>
 
 #include <gensio/gensio_types.h>
-#include <gensio/gensio_dllvisibility.h>
+#include <gensio/gensioosh_dllvisibility.h>
 
 /* Avoid having to include SCTP headers. */
 struct sctp_sndrcvinfo;
@@ -44,7 +44,6 @@ struct gensio_iod {
 enum gensio_iod_type {
     GESNIO_IOD_INVALID, /* Not used for anything the user will see. */
     GENSIO_IOD_SOCKET,
-    /* User cannot allocate these with add_iod().  See note in gensio_win.c */
     GENSIO_IOD_PIPE,
     GENSIO_IOD_DEV,
     GENSIO_IOD_FILE,
@@ -91,7 +90,7 @@ enum gensio_iod_type {
 };
 
 struct gensio_once {
-    bool called;
+    int called;
 };
 
 /* Used by open_listen_sockets() to return the opened sockets. */
@@ -565,10 +564,16 @@ struct gensio_os_funcs {
 
     /****** I/O Descriptors ******/
     /*
-     * Allocate an I/O descriptor for an fd.
+     * Allocate an I/O descriptor for an fd.  For pipe handles on
+     * Windows, the ... is an integer saying whether the IOD is
+     * readable or writeable, either GENSIO_IOD_READABLE or
+     * GENSIO_IOD_WRITEABLE.  You can pass this in on Unix, too, it
+     * will have no effect.
      */
     int (*add_iod)(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		   intptr_t fd, struct gensio_iod **iod);
+		   intptr_t fd, struct gensio_iod **iod, ...);
+#define GENSIO_IOD_WRITEABLE	0
+#define GENSIO_IOD_READABLE	1
 
     /*
      * Release an allocated I/O descriptor.  Note that close() will
@@ -826,15 +831,30 @@ struct gensio_os_funcs {
  * Called from os handlers, check for any handlers that may need to be
  * called.
  */
-GENSIO_DLL_PUBLIC
+GENSIOOSH_DLL_PUBLIC
 void gensio_os_proc_check_handlers(struct gensio_os_proc_data *data);
 
 #ifndef _WIN32
 #include <signal.h>
-GENSIO_DLL_PUBLIC
+GENSIOOSH_DLL_PUBLIC
 sigset_t *gensio_os_proc_unix_get_wait_sigset(struct gensio_os_proc_data *data);
 #endif
 
+struct gensio_os_cleanup_handler {
+    void (*cleanup)(struct gensio_os_cleanup_handler *h);
+    struct gensio_os_cleanup_handler *next;
+};
+
+/*
+ * Register a handler to be called when gensio_os_proc_cleanup() is
+ * called.  This does not free the data for "h", but it is
+ * automatically unlinked before the cleanup handler is called, so you
+ * may free it in the handler.
+ */
+GENSIOOSH_DLL_PUBLIC
+void gensio_register_os_cleanup_handler(struct gensio_os_funcs *o,
+					struct gensio_os_cleanup_handler *h);
+
 #ifdef __cplusplus
 }
 #endif
