@@ -22,11 +22,12 @@
 #define LOCK_THREAD_ID_EQUAL(a,b) ((a) == (b))
 #elif defined(USE_PTHREADS)
 #include <pthread.h>
+#include <assert.h>
 #define lock_type pthread_mutex_t
-#define LOCK_INIT(l) pthread_mutex_init(l, NULL)
-#define LOCK_DESTROY(l) pthread_mutex_destroy(l)
-#define LOCK(l) pthread_mutex_lock(l)
-#define UNLOCK(l) pthread_mutex_unlock(l)
+#define LOCK_INIT(l) assert(pthread_mutex_init(l, NULL) == 0)
+#define LOCK_DESTROY(l) assert(pthread_mutex_destroy(l)== 0)
+#define LOCK(l) assert(pthread_mutex_lock(l) == 0)
+#define UNLOCK(l) assert(pthread_mutex_unlock(l) == 0)
 #define LOCK_INITIALIZER PTHREAD_MUTEX_INITIALIZER
 #define lock_thread_type pthread_t
 #define LOCK_GET_THREAD_ID pthread_self()
