@@ -1563,9 +1563,11 @@ gensio_loadlib(struct gensio_os_funcs *o, const char *str)
 	return false;
     memcpy(name, str, len);
     name[len] = '\0';
-    if (strcmp(name, "tcp") == 0 || strcmp(name, "unix") == 0)
+    if (strcmp(name, "udp") == 0 || strcmp(name, "unixdgram") == 0)
+	strncpy(name, "dgram", sizeof(name));
+    else if (strcmp(name, "tcp") == 0 || strncmp(name, "unix", 4) == 0)
 	strncpy(name, "net", sizeof(name));
-    if (strcmp(name, "dev") == 0 || strcmp(name, "sdev") == 0)
+    else if (strcmp(name, "dev") == 0 || strcmp(name, "sdev") == 0)
 	strncpy(name, "serialdev", sizeof(name));
 
     return gensio_os_loadlib(o, name);
@@ -2481,6 +2483,31 @@ gensio_pparm_float(struct gensio_pparm_info *p,
     return 1;
 }
 
+int
+gensio_pparm_argv(struct gensio_pparm_info *p,
+		  const char *str, const char *key, const char *seps,
+		  int *argc, const char ***argv)
+{
+    const char *sval;
+    int rv = gensio_pparm_value(p, str, key, &sval);
+
+    if (!rv)
+	return 0;
+
+    if (!*sval) {
+	gensio_pparm_log(p, "no value given in parameter %s", str);
+	return -1;
+    }
+
+    rv = gensio_str_to_argv(p->o, sval, argc, argv, seps);
+    if (rv) {
+	gensio_pparm_log(p, "Error getting argv list for %s: %s", str,
+			 gensio_err_to_str(rv));
+	return -1;
+    }
+    return 1;
+}
+
 void
 gensio_pparm_unknown_parm(struct gensio_pparm_info *p,
 			  const char *arg)
