@@ -13,6 +13,7 @@
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include <netioapi.h>
+#include <afunix.h>
 #else
 #include <errno.h>
 #include <sys/types.h>
@@ -30,6 +31,9 @@
 #include <sys/un.h>
 #endif
 
+#define SIZEOF_SOCKADDR_UN_HEADER \
+    (sizeof(struct sockaddr_un) - \
+     sizeof(((struct sockaddr_un *) 0)->sun_path))
 
 #include <gensio/gensio.h>
 #include <gensio/gensio_os_funcs.h>
@@ -171,9 +175,7 @@ gensio_addr_addrinfo_create(struct gensio_os_funcs *o,
 #ifdef AF_INET6
     struct sockaddr_in6 s6 = { .sin6_family = AF_INET6 };
 #endif
-#if HAVE_UNIX
     struct sockaddr_un su = { .sun_family = AF_UNIX };
-#endif
     struct sockaddr *s;
     unsigned int slen;
     struct gensio_addr_addrinfo *a;
@@ -210,16 +212,12 @@ gensio_addr_addrinfo_create(struct gensio_os_funcs *o,
 #endif
 
     case GENSIO_NETTYPE_UNIX:
-#if HAVE_UNIX
 	if (len > sizeof(su.sun_path) - 1)
 	    return GE_TOOBIG;
 	memcpy(su.sun_path, iaddr, len);
 	s = (struct sockaddr *) &su;
 	slen = sizeof(su);
 	break;
-#else
-	return GE_NOTSUP;
-#endif
 
     default:
 	return GE_INVAL;
@@ -312,7 +310,6 @@ gensio_addr_addrinfo_get_data(const struct gensio_addr *aaddr,
 	break;
     }
 
-#if HAVE_UNIX
     case AF_UNIX: {
 	struct sockaddr_un *su = (struct sockaddr_un *) addr->curr->ai_addr;
 	data = su->sun_path;
@@ -320,7 +317,6 @@ gensio_addr_addrinfo_get_data(const struct gensio_addr *aaddr,
 	break;
     }
 
-#endif
     default:
 	data = &dummy;
 	len = 0;
@@ -399,16 +395,15 @@ sockaddr_equal(const struct sockaddr *a1, socklen_t l1,
 	break;
 #endif
 
-#if HAVE_UNIX
     case AF_UNIX:
 	{
 	    struct sockaddr_un *s1 = (struct sockaddr_un *) a1;
 	    struct sockaddr_un *s2 = (struct sockaddr_un *) a2;
-	    if (strcmp(s1->sun_path, s2->sun_path) != 0)
+	    if (strncmp(s1->sun_path, s2->sun_path,
+			sizeof(s1->sun_path)) != 0)
 		return false;
 	}
 	break;
-#endif
 
     default:
 	/* Unknown family. */
@@ -478,12 +473,10 @@ gensio_sockaddr_to_str(const struct sockaddr *addr, int flags,
 			ifbuf,
 			ntohs(a6->sin6_port));
 #endif
-#if HAVE_UNIX
     } else if (addr->sa_family == AF_UNIX) {
 	struct sockaddr_un *au = (struct sockaddr_un *) addr;
 
 	gensio_pos_snprintf(buf, buflen, pos, "unix,%s", au->sun_path);
-#endif
     } else if (addr->sa_family == GENSIO_AF_IFINDEX) {
 	struct sockaddr *as = (struct sockaddr *) addr;
 	unsigned int *iptr = (unsigned int *) as->sa_data;
@@ -539,10 +532,9 @@ gensio_addr_addrinfo_to_str_all(const struct gensio_addr *aaddr,
 }
 
 static int
-gensio_scan_unixaddr(struct gensio_os_funcs *o, const char *str,
+gensio_scan_unixaddr(struct gensio_os_funcs *o, int socktype, const char *str,
 		     struct gensio_addr **raddr)
 {
-#if HAVE_UNIX
     struct sockaddr_un *saddr;
     struct gensio_addr *addr = NULL;
     struct addrinfo *ai;
@@ -552,7 +544,8 @@ gensio_scan_unixaddr(struct gensio_os_funcs *o, const char *str,
     if (len >= sizeof(saddr->sun_path) - 1)
 	return GE_TOOBIG;
 
-    addr = gensio_addr_addrinfo_make(o, sizeof(socklen_t) + len + 1, false);
+    addr = gensio_addr_addrinfo_make(o, SIZEOF_SOCKADDR_UN_HEADER + len + 1,
+				     false);
     if (!addr)
 	return GE_NOMEM;
 
@@ -561,16 +554,13 @@ gensio_scan_unixaddr(struct gensio_os_funcs *o, const char *str,
     saddr->sun_family = AF_UNIX;
     memcpy(saddr->sun_path, str, len);
     ai->ai_family = AF_UNIX;
-    ai->ai_socktype = SOCK_STREAM;
-    ai->ai_addrlen = sizeof(socklen_t) + len + 1;
+    ai->ai_socktype = socktype;
+    ai->ai_addrlen = SIZEOF_SOCKADDR_UN_HEADER + len + 1;
     ai->ai_addr = (struct sockaddr *) saddr;
 
     *raddr = addr;
 
     return 0;
-#else
-    return GE_NOTSUP;
-#endif
 }
 
 #ifdef _MSC_VER
@@ -727,6 +717,8 @@ gensio_addr_addrinfo_scan_ips(struct gensio_os_funcs *o, const char *str,
     bool first = true, portset = false;
     int rv = 0, socktype, protocol;
     int bflags = AI_ADDRCONFIG;
+    struct sockaddr_storage iaddr;
+    char *end;
 
     switch (gprotocol) {
     case GENSIO_NET_PROTOCOL_UNSPEC:
@@ -753,8 +745,18 @@ gensio_addr_addrinfo_scan_ips(struct gensio_os_funcs *o, const char *str,
 	return GE_NOTSUP;
 #endif
 
+    case GENSIO_NET_PROTOCOL_UNIX_DGRAM:
+	socktype = SOCK_DGRAM;
+	goto do_unix;
+#ifdef SOCK_SEQPACKET
+    case GENSIO_NET_PROTOCOL_UNIX_SEQPACKET:
+	socktype = SOCK_SEQPACKET;
+	goto do_unix;
+#endif
     case GENSIO_NET_PROTOCOL_UNIX:
-	rv = gensio_scan_unixaddr(o, str, raddr);
+	socktype = SOCK_STREAM;
+    do_unix:
+	rv = gensio_scan_unixaddr(o, socktype, str, raddr);
 	if (!rv && is_port_set)
 	    *is_port_set = false;
 	return rv;
@@ -869,7 +871,33 @@ gensio_addr_addrinfo_scan_ips(struct gensio_os_funcs *o, const char *str,
 	 */
 	if (!ip && notype)
 	    rflags |= AI_V4MAPPED;
+#endif
 
+	/*
+	 * If the host address is a valid IP address, skip the
+	 * nameserver lookup and just tell getaddrinfo to convert the
+	 * numeric value.
+	 */
+	if (ip) {
+	    if (family == AF_INET && inet_pton(AF_INET, ip, &iaddr) == 1)
+		rflags |= AI_NUMERICHOST;
+#ifdef AF_INET6
+	    else if (family == AF_INET6) {
+		if (inet_pton(AF_INET6, ip, &iaddr) == 1)
+		    rflags |= AI_NUMERICHOST;
+		else if (rflags & AI_V4MAPPED &&
+			inet_pton(AF_INET, ip, &iaddr) == 1)
+		    rflags |= AI_NUMERICHOST;
+	    }
+#endif
+	}
+	/*
+	 * If the port string is a number, just use the number and don't
+	 * do the nameserver lookup for the service.
+	 */
+	if (port && strtoul(port, &end, 10) < 65536 && *end == '\0')
+	    rflags |= AI_NUMERICSERV;
+#ifdef AF_INET6
     redo_getaddrinfo:
 #endif
 	memset(&hints, 0, sizeof(hints));
@@ -1166,9 +1194,7 @@ gensio_addr_addrinfo_get_nettype(const struct gensio_addr *aaddr)
     switch (addr->curr->ai_addr->sa_family) {
     case AF_INET: return GENSIO_NETTYPE_IPV4;
     case AF_INET6: return GENSIO_NETTYPE_IPV6;
-#if HAVE_UNIX
     case AF_UNIX: return GENSIO_NETTYPE_UNIX;
-#endif
     default: return GENSIO_NETTYPE_UNSPEC;
     }
 }
