@@ -12,9 +12,10 @@
 #include <gensio/gensio_time.h>
 
 #ifdef _WIN32
-#define DIRSEP '\\'
+/* On Windows you can use / or \. */
+#define DIRSEPS "\\/"
 #else
-#define DIRSEP '/'
+#define DIRSEPS "/"
 #endif
 
 struct gensio_certauth_filter_data {
@@ -548,11 +549,18 @@ struct certauth_filter {
 
 #define filter_to_certauth(v) ((struct certauth_filter *) \
 			       gensio_filter_get_user_data(v))
+static void certauth_lock(struct certauth_filter *sfilter);
+static void certauth_unlock(struct certauth_filter *sfilter);
 
+/*
+ * This function releases and reclaims the lock, so it can only be
+ * called in places where this is ok.
+ */
 static void
 gca_vlog(struct certauth_filter *f, enum gensio_log_levels l,
 	 bool do_ssl_err, char *fmt, va_list ap)
 {
+    certauth_unlock(f);
     if (do_ssl_err) {
 	char buf[256], buf2[200];
 	unsigned long ssl_err = ERR_get_error();
@@ -562,11 +570,12 @@ gca_vlog(struct certauth_filter *f, enum gensio_log_levels l,
 
 	ERR_error_string_n(ssl_err, buf2, sizeof(buf2));
 	snprintf(buf, sizeof(buf), "certauth: %s: %s", fmt, buf2);
-	gensio_vlog(f->o, l, buf, ap);
+	gensio_filter_vlog(f->filter, l, buf, ap);
     } else {
     no_ssl_err:
-	gensio_vlog(f->o, l, fmt, ap);
+	gensio_filter_vlog(f->filter, l, fmt, ap);
     }
+    certauth_lock(f);
 }
 
 
@@ -1333,9 +1342,9 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout,
 		gca_log_err(sfilter,
 		    "Got server data while client waiting in challenge delay");
 		sfilter->pending_err = GE_NOTREADY;
-		break;
+		goto finish_result;
 	    }
-	    goto out;
+	    goto out_inprogress;
 	}
 
 	set_digest(sfilter);
@@ -1725,18 +1734,18 @@ static int
 certauth_ul_write(struct gensio_filter *filter,
 		  gensio_ul_filter_data_handler handler, void *cb_data,
 		  gensiods *rcount,
-		  const struct gensio_sg *sg, gensiods sglen,
+		  const struct gensio_sg *isg, gensiods sglen,
 		  const char *const *auxdata)
 {
     struct certauth_filter *sfilter = filter_to_certauth(filter);
     int rv = 0;
 
     certauth_lock(sfilter);
-    if (sg) {
+    if (isg) {
 	if (sfilter->state != CERTAUTH_PASSTHROUGH || sfilter->pending_err)
 	    rv = GE_NOTREADY;
 	else
-	    rv = handler(cb_data, rcount, sg, sglen, auxdata);
+	    rv = handler(cb_data, rcount, isg, sglen, auxdata);
 	if (rv)
 	    goto out_unlock;
     }
@@ -2441,7 +2450,7 @@ certauth_filter_control(struct gensio_filter *filter, bool get, int op,
 	store = X509_STORE_new();
 	if (!store)
 	    return GE_NOMEM;
-	if (data[strlen(data) - 1] == DIRSEP)
+	if (strchr(DIRSEPS, data[strlen(data) - 1]))
 	    CApath = data;
 	else
 	    CAfile = data;
@@ -3052,7 +3061,7 @@ gensio_certauth_filter_alloc(struct gensio_certauth_filter_data *data,
     if (data->CAfilepath && data->CAfilepath[0]) {
 	char *CAfile = NULL, *CApath = NULL;
 
-	if (data->CAfilepath[strlen(data->CAfilepath) - 1] == DIRSEP)
+	if (strchr(DIRSEPS, data->CAfilepath[strlen(data->CAfilepath) - 1]))
 	    CApath = data->CAfilepath;
 	else
 	    CAfile = data->CAfilepath;
