@@ -208,13 +208,29 @@ gensio_scan_network_port(struct gensio_os_funcs *o, const char *str,
 #endif
     }
 
-    if (strncmp(str, "unix,", 4) == 0 ||
-		(rargs && strncmp(str, "unix(", 4) == 0)) {
+    if (strncmp(str, "unix,", 5) == 0 ||
+		(rargs && strncmp(str, "unix(", 5) == 0)) {
 	if (family != AF_UNSPEC)
 	    return GE_INVAL;
 	str += 4;
     handle_unix:
 	protocol = GENSIO_NET_PROTOCOL_UNIX;
+#ifdef SOCK_SEQPACKET
+    } else if (strncmp(str, "unixseq,", 8) == 0 ||
+		(rargs && strncmp(str, "unixseq(", 8) == 0)) {
+	if (family != AF_UNSPEC)
+	    return GE_INVAL;
+	str += 7;
+    handle_unix_seqpacket:
+	protocol = GENSIO_NET_PROTOCOL_UNIX_SEQPACKET;
+#endif
+    } else if (strncmp(str, "unixdgram,", 10) == 0 ||
+		(rargs && strncmp(str, "unixdgram(", 10) == 0)) {
+	if (family != AF_UNSPEC)
+	    return GE_INVAL;
+	str += 9;
+    handle_unix_dgram:
+	protocol = GENSIO_NET_PROTOCOL_UNIX_DGRAM;
     } else if (strncmp(str, "tcp,", 4) == 0 ||
 		(rargs && strncmp(str, "tcp(", 4) == 0)) {
 	str += 3;
@@ -239,6 +255,12 @@ gensio_scan_network_port(struct gensio_os_funcs *o, const char *str,
 	switch (*rprotocol) {
 	case GENSIO_NET_PROTOCOL_UNIX:
 	    goto handle_unix;
+	case GENSIO_NET_PROTOCOL_UNIX_DGRAM:
+	    goto handle_unix_dgram;
+#ifdef SOCK_SEQPACKET
+	case GENSIO_NET_PROTOCOL_UNIX_SEQPACKET:
+	    goto handle_unix_seqpacket;
+#endif
 	case GENSIO_NET_PROTOCOL_TCP:
 	    goto handle_tcp;
 	case GENSIO_NET_PROTOCOL_UDP:
@@ -308,7 +330,9 @@ gensio_os_scan_netaddr(struct gensio_os_funcs *o, const char *str, bool listen,
     rv = o->addr_scan_ips(o, str, listen, AF_UNSPEC,
 			  protocol, &is_port_set, true, &addr);
     if (!rv && !listen && !is_port_set &&
-		protocol != GENSIO_NET_PROTOCOL_UNIX) {
+		protocol != GENSIO_NET_PROTOCOL_UNIX &&
+		protocol != GENSIO_NET_PROTOCOL_UNIX_DGRAM &&
+		protocol != GENSIO_NET_PROTOCOL_UNIX_SEQPACKET) {
 	gensio_addr_free(addr);
 	rv = GE_INVAL;
     } else if (!rv) {
@@ -2421,8 +2445,8 @@ struct gensio_unix_termios {
     g_termios curr_termios;
     bool break_set;
 #if HAVE_DECL_TIOCSRS485
-    bool rs485_applied;
     struct serial_rs485 rs485;
+    struct serial_rs485 orig_rs485;
 #endif
 };
 
@@ -2559,6 +2583,9 @@ gensio_unix_setup_termios(struct gensio_os_funcs *o, int fd,
 
     t->orig_termios = t->curr_termios;
     ioctl(fd, TIOCMGET, &t->orig_mctl);
+#if HAVE_DECL_TIOCSRS485
+    ioctl(fd, TIOCGRS485, &t->orig_rs485);
+#endif
 
     s_cfmakeraw(&t->curr_termios);
 
@@ -2579,6 +2606,9 @@ gensio_unix_cleanup_termios(struct gensio_os_funcs *o,
 {
     if (!*it)
 	return;
+#if HAVE_DECL_TIOCSRS485
+    ioctl(fd, TIOCSRS485, &(*it)->orig_rs485);
+#endif
     ioctl(fd, TIOCMSET, &(*it)->orig_mctl);
     set_termios(fd, &(*it)->orig_termios);
     o->free(o, *it);
@@ -3006,15 +3036,12 @@ gensio_unix_termios_control(struct gensio_os_funcs *o, int op, bool get,
 	if (rv) {
 	    rv = gensio_os_err_to_err(o, errno);
 #if HAVE_DECL_TIOCSRS485
+	} else if (t->rs485.flags & SER_RS485_ENABLED) {
+	    if (ioctl(fd, TIOCSRS485, &t->rs485) < 0)
+		rv = gensio_os_err_to_err(o, errno);
 	} else {
-	    bool enabled = !!(t->rs485.flags & SER_RS485_ENABLED);
-
-	    if (enabled != t->rs485_applied) {
-		if (ioctl(fd, TIOCSRS485, &t->rs485) < 0)
-		    rv = gensio_os_err_to_err(o, errno);
-		else
-		    t->rs485_applied = enabled;
-	    }
+	    /* If the disable fails, we don't care. */
+	    ioctl(fd, TIOCSRS485, &t->rs485);
 #endif
 	}
 	break;
