@@ -16,6 +16,7 @@
 #ifdef _WIN32
 #include <winsock2.h>
 #include <ws2tcpip.h>
+#include <afunix.h>
 typedef int taddrlen;
 typedef int sockret;
 #define sock_errno WSAGetLastError()
@@ -69,6 +70,10 @@ typedef ssize_t sockret;
 #define AI_V4MAPPED 0
 #endif
 
+#define SIZEOF_SOCKADDR_UN_HEADER \
+    (sizeof(struct sockaddr_un) - \
+     sizeof(((struct sockaddr_un *) 0)->sun_path))
+
 struct gensio_stdsock_info {
     int protocol;
     int family;
@@ -113,7 +118,7 @@ struct gensio_listen_scan_info {
  * opened on all addresses.  Errors besides GE_ADDRINUSE should be
  * treated as immediate errors, something else went wrong.
  */
-static int gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
+static int gensio_setup_listen_socket(struct gensio_os_funcs *o,
 			       int family, int socktype, int protocol,
 			       int flags,
 			       struct sockaddr *addr, socklen_t addrlen,
@@ -270,7 +275,7 @@ gensio_os_sctp_open_sockets(struct gensio_os_funcs *o,
 	if (fds)
 	    memcpy(tfds, fds, sizeof(*tfds) * i);
 
-	rv = gensio_setup_listen_socket(o, true, rp->ai_family,
+	rv = gensio_setup_listen_socket(o, rp->ai_family,
 					SOCK_STREAM, IPPROTO_SCTP, rp->ai_flags,
 					rp->ai_addr, rp->ai_addrlen,
 					call_b4_listen, data, opensock_flags,
@@ -908,7 +913,17 @@ gensio_stdsock_recvfrom(struct gensio_iod *iod,
     rv = recvfrom(o->iod_get_fd(iod), buf, buflen, flags, ai->ai_addr, &len);
 #endif
     if (rv >= 0) {
-	ai->ai_addrlen = len;
+	if (len == 0) {
+	    /*
+	     * This happens when receiving an AF_UNIX datagram socket
+	     * where the other end isn't bound.  Just create a socket
+	     * with the family set and no path.
+	     */
+	    ai->ai_addrlen = sizeof(ai->ai_addr->sa_family);
+	    ai->ai_addr->sa_family = AF_UNIX;
+	} else {
+	    ai->ai_addrlen = len;
+	}
 	ai->ai_family = ai->ai_addr->sa_family;
     } else {
 	if (sock_errno == SOCK_EINTR)
@@ -1162,6 +1177,18 @@ gensio_stdsock_socket_open(struct gensio_os_funcs *o,
 	socktype = SOCK_STREAM;
 	break;
 
+    case GENSIO_NET_PROTOCOL_UNIX_DGRAM:
+	sockproto = 0;
+	socktype = SOCK_DGRAM;
+	break;
+
+#ifdef SOCK_SEQPACKET
+    case GENSIO_NET_PROTOCOL_UNIX_SEQPACKET:
+	sockproto = 0;
+	socktype = SOCK_SEQPACKET;
+	break;
+#endif
+
     case GENSIO_NET_PROTOCOL_UDP:
 	sockproto = 0;
 	socktype = SOCK_DGRAM;
@@ -1283,6 +1310,10 @@ gensio_stdsock_socket_set_setup(struct gensio_iod *iod,
 	case GENSIO_NET_PROTOCOL_TCP:
 	case GENSIO_NET_PROTOCOL_UDP:
 	case GENSIO_NET_PROTOCOL_UNIX:
+	case GENSIO_NET_PROTOCOL_UNIX_DGRAM:
+#ifdef SOCK_SEQPACKET
+	case GENSIO_NET_PROTOCOL_UNIX_SEQPACKET:
+#endif
 	    ai = gensio_addr_addrinfo_get_curr(bindaddr);
 	    if (bind(fd, ai->ai_addr, ai->ai_addrlen) == -1)
 		return gensio_os_err_to_err(o, sock_errno);
@@ -1428,7 +1459,7 @@ gensio_stdsock_close_socket(struct gensio_iod *iod, bool retry, bool force)
 #else
     err = close_socket(o, o->iod_get_fd(iod));
 #endif
-    if (err != GE_INPROGRESS)
+    if (gsi && err != GE_INPROGRESS)
 	o->free(o, gsi);
     return err;
 }
@@ -2059,9 +2090,7 @@ gensio_stdsock_open_listen_sockets(struct gensio_os_funcs *o,
     family = AF_INET;
 #endif
 
-#if defined(AF_INET6) || HAVE_UNIX
  restart:
-#endif
     for (rp = ai; rp != NULL; rp = rp->ai_next) {
 	if (family != rp->ai_family)
 	    continue;
@@ -2075,8 +2104,7 @@ gensio_stdsock_open_listen_sockets(struct gensio_os_funcs *o,
 	if (sockaddr_in_list_b4(rp, ai))
 	    continue;
 
-	rv = gensio_setup_listen_socket(o, rp->ai_socktype == SOCK_STREAM,
-					rp->ai_family, rp->ai_socktype,
+	rv = gensio_setup_listen_socket(o, rp->ai_family, rp->ai_socktype,
 					rp->ai_protocol, rp->ai_flags,
 					rp->ai_addr, rp->ai_addrlen,
 					call_b4_listen, data,
@@ -2095,12 +2123,10 @@ gensio_stdsock_open_listen_sockets(struct gensio_os_funcs *o,
 	goto restart;
     }
 #endif
-#if HAVE_UNIX
     if (family == AF_INET) {
 	family = AF_UNIX;
 	goto restart;
     }
-#endif
 
     if (curr_fd == 0) {
 	o->free(o, fds);
@@ -2146,7 +2172,7 @@ family_is_inet(int family)
 }
 
 static int
-gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
+gensio_setup_listen_socket(struct gensio_os_funcs *o,
 			   int family, int socktype, int sockproto, int flags,
 			   struct sockaddr *addr, socklen_t addrlen,
 			   int (*call_b4_listen)(struct gensio_iod *, void *),
@@ -2162,8 +2188,16 @@ gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
     struct gensio_iod *iod = NULL;
     int protocol;
     struct gensio_stdsock_info *gsi = NULL;
+    bool do_listen = true;
 
-    if (family == AF_UNIX)
+    if (family == AF_UNIX && socktype == SOCK_DGRAM) {
+	protocol = GENSIO_NET_PROTOCOL_UNIX_DGRAM;
+	do_listen = false;
+#ifdef SOCK_SEQPACKET
+    } else if (family == AF_UNIX && socktype == SOCK_SEQPACKET) {
+	protocol = GENSIO_NET_PROTOCOL_UNIX_SEQPACKET;
+#endif
+    }else if (family == AF_UNIX)
 	protocol = GENSIO_NET_PROTOCOL_UNIX;
     else if (sockproto == IPPROTO_SCTP)
 	protocol = GENSIO_NET_PROTOCOL_SCTP;
@@ -2173,9 +2207,10 @@ gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
 	protocol = GENSIO_NET_PROTOCOL_TCP;
     else if (sockproto == IPPROTO_TCP)
 	protocol = GENSIO_NET_PROTOCOL_TCP;
-    else if (sockproto == IPPROTO_UDP)
+    else if (sockproto == IPPROTO_UDP) {
 	protocol = GENSIO_NET_PROTOCOL_UDP;
-    else
+	do_listen = false;
+    } else
 	return GE_INVAL;
 
     rv = gensio_sockaddr_get_port(addr, &port);
@@ -2202,7 +2237,7 @@ gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
 
     if (opensock_flags & GENSIO_OPENSOCK_REUSEADDR) {
 	switch (family) {
-#if HAVE_UNIX
+#if HAVE_UNIX || defined(_WIN32)
 	case AF_UNIX: {
 	    /* We remove an existing socket with reuseaddr and AF_UNIX. */
 	    struct sockaddr_un *unaddr = (struct sockaddr_un *) addr;
@@ -2213,10 +2248,14 @@ gensio_setup_listen_socket(struct gensio_os_funcs *o, bool do_listen,
 	     * in the unix(7) man page on Linux for details.
 	     */
 	    assert(addrlen <= sizeof(*unaddr));
-	    memcpy(unpath, unaddr->sun_path, addrlen - sizeof(sa_family_t));
-	    unpath[addrlen - sizeof(sa_family_t)] = '\0';
+	    memcpy(unpath, unaddr->sun_path, addrlen-SIZEOF_SOCKADDR_UN_HEADER);
+	    unpath[addrlen - SIZEOF_SOCKADDR_UN_HEADER] = '\0';
 
+#if HAVE_UNIX
 	    unlink(unpath);
+#else
+	    DeleteFile(unpath);
+#endif
 	    /*
 	     * Ignore errors, it may not exist, and we'll get errors
 	     * later on problems.
