@@ -214,7 +214,7 @@ i_wait_for_waiter_timeout(waiter_t *waiter, unsigned int count,
     w.prev = NULL;
     w.count = count;
 
-    pthread_mutex_lock(&waiter->lock);
+    LOCK(&waiter->lock);
     waiter->wts.next->prev = &w;
     w.next = waiter->wts.next;
     waiter->wts.next = &w;
@@ -232,7 +232,7 @@ i_wait_for_waiter_timeout(waiter_t *waiter, unsigned int count,
 	}
     }
     while (w.count > 0) {
-	pthread_mutex_unlock(&waiter->lock);
+	UNLOCK(&waiter->lock);
 	if (intr)
 	    err = sel_select_intr_sigmask(waiter->sel,
 					  wake_thread_send_sig_waiter,
@@ -247,7 +247,7 @@ i_wait_for_waiter_timeout(waiter_t *waiter, unsigned int count,
 	else
 	    err = 0;
 	/* lock may affect errno, delay it until here. */
-	pthread_mutex_lock(&waiter->lock);
+	LOCK(&waiter->lock);
 	if (err)
 	    break;
     }
@@ -263,7 +263,7 @@ i_wait_for_waiter_timeout(waiter_t *waiter, unsigned int count,
 	 */
 	i_wake_waiter(waiter, count - w.count);
     }
-    pthread_mutex_unlock(&waiter->lock);
+    UNLOCK(&waiter->lock);
 
     return err;
 }
@@ -271,9 +271,9 @@ i_wait_for_waiter_timeout(waiter_t *waiter, unsigned int count,
 static void
 wake_waiter(waiter_t *waiter)
 {
-    pthread_mutex_lock(&waiter->lock);
+    LOCK(&waiter->lock);
     i_wake_waiter(waiter, 1);
-    pthread_mutex_unlock(&waiter->lock);
+    UNLOCK(&waiter->lock);
 }
 
 #else /* USE_PTHREADS */
@@ -1699,6 +1699,10 @@ struct gensio_os_proc_data {
     sigset_t wait_sigs; /* Signal mask to use when waiting. */
     sigset_t check_sigs; /* Signals we are checking for. */
 
+#define MAX_HANDLE_SIGS 10
+    int handle_sigs[MAX_HANDLE_SIGS];
+    unsigned int num_handle_sigs;
+
     struct sigaction old_wakesig;
 
     struct sigaction old_sigchld;
@@ -1796,7 +1800,7 @@ gensio_os_thread_setup(struct gensio_os_funcs *o)
     }
 
     if (wake_sig) {
-	memset(&sigdo, 7, sizeof(sigdo));
+	memset(&sigdo, 0, sizeof(sigdo));
 	sigdo.sa_handler = handle_wakesig;
 	sigdo.sa_flags = 0;
 	rv = sigaction(wake_sig, &sigdo, NULL);
@@ -1812,6 +1816,33 @@ gensio_os_thread_setup(struct gensio_os_funcs *o)
     return rv;
 }
 
+static void
+gensio_os_proc_handle_sig(int sig, void *sdata)
+{
+    struct gensio_os_proc_data *data = sdata;
+
+    switch(sig) {
+    case SIGCHLD:
+	data->got_sigchld = true;
+	break;
+    case SIGQUIT:
+    case SIGTERM:
+    case SIGINT:
+	data->got_term_sig = true;
+	break;
+    case SIGHUP:
+	data->got_reload_sig = true;
+	break;
+#if HAVE_DECL_SIGWINCH
+    case SIGWINCH:
+	data->got_winch_sig = true;
+	break;
+#endif
+    default:
+	assert(0);
+    }
+}
+
 int
 gensio_os_proc_setup(struct gensio_os_funcs *o,
 		     struct gensio_os_proc_data **rdata)
@@ -1890,6 +1921,9 @@ gensio_os_proc_setup(struct gensio_os_funcs *o,
     }
 
     LOCK_INIT(&data->handler_lock);
+    data->handle_sigs[data->num_handle_sigs++] = SIGCHLD;
+
+    sel_set_handle_sig(data->handle_sigs, gensio_os_proc_handle_sig, data);
 
     *rdata = data;
     proc_setup = true;
@@ -1916,8 +1950,29 @@ check_for_sigpending(sigset_t *check_for)
 {
 #ifdef HAVE_SIGTIMEDWAIT
     static struct timespec zerotime = { 0, 0 };
-    return sigtimedwait(check_for, NULL, &zerotime);
+    int rv;
+ retry:
+    rv = sigtimedwait(check_for, NULL, &zerotime);
+    if (rv < 0) {
+	if (errno == EINTR)
+	    goto retry;
+	if (errno == EAGAIN)
+	    return 0;
+	assert(0);
+    }
+    return rv;
 #else
+    return 0;
+#endif
+#if 0
+    /*
+     * The below code doesn't work, it can race with a signal handler
+     * and lock up.  This is only an issue on MacOS, whose lack of
+     * sigtimedwait is ridiculous.  You might be able to do this with
+     * kevent, but that would be expensive.  Maybe it's not necessary
+     * on MacOS, maybe pselect will handle signals on all calls, even
+     * if there's no wait.
+     */
 #ifdef NSIG
     /* Nothing to do */
 #elif defined(_NSIG)
@@ -2003,28 +2058,8 @@ gensio_os_proc_check_handlers(struct gensio_os_proc_data *data)
      * missing a signal on a really busy system, check for signals
      * here.
      */
-    while ((sig = check_for_sigpending(&data->check_sigs)) > 0) {
-	switch(sig) {
-	case SIGCHLD:
-	    data->got_sigchld = true;
-	    break;
-	case SIGQUIT:
-	case SIGTERM:
-	case SIGINT:
-	    data->got_term_sig = true;
-	    break;
-	case SIGHUP:
-	    data->got_reload_sig = true;
-	    break;
-#if HAVE_DECL_SIGWINCH
-	case SIGWINCH:
-	    data->got_winch_sig = true;
-	    break;
-#endif
-	default:
-	    assert(0);
-	}
-    }
+    while ((sig = check_for_sigpending(&data->check_sigs)) > 0)
+	gensio_os_proc_handle_sig(sig, data);
     if (data->got_term_sig) {
 	data->got_term_sig = false;
 	data->term_handler(data->term_handler_data);
@@ -2100,6 +2135,10 @@ gensio_os_proc_register_term_handler(struct gensio_os_proc_data *data,
 	goto out_err;
     }
 
+    data->handle_sigs[data->num_handle_sigs++] = SIGINT;
+    data->handle_sigs[data->num_handle_sigs++] = SIGQUIT;
+    data->handle_sigs[data->num_handle_sigs++] = SIGTERM;
+
     sigdelset(&data->wait_sigs, SIGINT);
     sigdelset(&data->wait_sigs, SIGQUIT);
     sigdelset(&data->wait_sigs, SIGTERM);
@@ -2145,6 +2184,7 @@ int gensio_os_proc_register_reload_handler(struct gensio_os_proc_data *data,
 	err = errno;
 	goto out_err;
     }
+    data->handle_sigs[data->num_handle_sigs++] = SIGHUP;
     sigdelset(&data->wait_sigs, SIGHUP);
     sigaddset(&data->check_sigs, SIGHUP);
     data->reload_sig_set = true;
@@ -2196,6 +2236,7 @@ gensio_os_proc_register_winsize_handler(struct gensio_os_proc_data *data,
 	err = gensio_os_err_to_err(data->o, errno);
 	goto out_unlock;
     }
+    data->handle_sigs[data->num_handle_sigs++] = SIGWINCH;
     sigdelset(&data->wait_sigs, SIGWINCH);
     data->winch_sig_set = true;
     /*
