@@ -18,16 +18,17 @@
 
 static int
 xlt_gensio_alloc(struct gensio *child, const char *const args[],
-		   struct gensio_os_funcs *o,
-		   gensio_event cb, void *user_data,
-		   struct gensio **net)
+		 struct gensio_os_funcs *o,
+		 gensio_event cb, void *user_data,
+		 struct gensio **net)
 {
     int err;
     struct gensio_filter *filter;
     struct gensio_ll *ll;
     struct gensio *io;
+    GENSIO_DECLARE_PPGENSIO(p, o, cb, "xlt", user_data);
 
-    err = gensio_xlt_filter_alloc(o, args, &filter);
+    err = gensio_xlt_filter_alloc(&p, o, args, &filter);
     if (err)
 	return err;
 
@@ -62,7 +63,8 @@ str_to_xlt_gensio(const char *str, const char * const args[],
     int err;
     struct gensio *io2;
 
-    err = str_to_gensio(str, o, NULL, NULL, &io2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio(str, o, cb, user_data, &io2);
     if (err)
 	return err;
 
@@ -77,6 +79,8 @@ struct xltna_data {
     struct gensio_accepter *acc;
     const char **args;
     struct gensio_os_funcs *o;
+    gensio_accepter_event cb;
+    void *user_data;
 };
 
 static void
@@ -103,8 +107,10 @@ xltna_new_child(void *acc_data, void **finish_data,
 		  struct gensio_filter **filter)
 {
     struct xltna_data *nadata = acc_data;
+    GENSIO_DECLARE_PPACCEPTER(p, nadata->o, nadata->cb, "xlt",
+			      nadata->user_data);
 
-    return gensio_xlt_filter_alloc(nadata->o, nadata->args, filter);
+    return gensio_xlt_filter_alloc(&p, nadata->o, nadata->args, filter);
 }
 
 static int
@@ -158,6 +164,8 @@ xlt_gensio_accepter_alloc(struct gensio_accepter *child,
     }
 
     nadata->o = o;
+    nadata->cb = cb;
+    nadata->user_data = user_data;
 
     err = gensio_gensio_accepter_alloc(child, o, "xlt", cb, user_data,
 				       gensio_gensio_acc_xlt_cb, nadata,
@@ -186,7 +194,8 @@ str_to_xlt_gensio_accepter(const char *str, const char * const args[],
     int err;
     struct gensio_accepter *acc2 = NULL;
 
-    err = str_to_gensio_accepter(str, o, NULL, NULL, &acc2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio_accepter(str, o, cb, user_data, &acc2);
     if (!err) {
 	err = xlt_gensio_accepter_alloc(acc2, args, o, cb, user_data, acc);
 	if (err)
