@@ -13,6 +13,7 @@
    routines registered with it. */
 
 #include "config.h"
+
 #include <gensio/selector.h>
 
 #include <sys/time.h>
@@ -25,12 +26,33 @@
 #include <syslog.h>
 #include <string.h>
 #include <assert.h>
+
+/*
+ * Older BSD systems have kevent, but don't have EVFILT_EXCEPT, which
+ * we need for TCP urgent data to work.  So just fall back to select
+ * in that case.
+ */
+#ifdef HAVE_KEVENT
+#include <sys/event.h>
+#ifdef EVFILT_EXCEPT
+#define USE_KEVENT
+#endif
+#endif
+
 #ifdef HAVE_EPOLL_PWAIT
 #include <sys/epoll.h>
+#define SEL_FD_ADD EPOLL_CTL_ADD
+#define SEL_FD_DEL EPOLL_CTL_DEL
+#define SEL_FD_MOD EPOLL_CTL_MOD
+#elif defined(USE_KEVENT)
+#define SEL_FD_ADD 1
+#define SEL_FD_DEL 2
+#define SEL_FD_MOD 3
 #else
-#define EPOLL_CTL_ADD 0
-#define EPOLL_CTL_DEL 0
-#define EPOLL_CTL_MOD 0
+/* These don't do anything if using pselect(). */
+#define SEL_FD_ADD 0
+#define SEL_FD_DEL 0
+#define SEL_FD_MOD 0
 #endif
 #include "errtrig.h"
 
@@ -96,6 +118,9 @@ typedef struct fd_control_s
     char write_enabled;
     char except_enabled;
 
+    /* On kevent, some I/O types don't handle EXCEPT, track that. */
+    enum { SEL_RDWR = 0, SEL_RDWR_NOEXC } iodir;
+
 #ifdef HAVE_EPOLL_PWAIT
     /* See the comment in process_fds_epoll() on the use of this. */
     uint32_t saved_events;
@@ -179,9 +204,17 @@ typedef struct sel_wait_list_s
     sel_send_sig_cb send_sig;
     void            *send_sig_cb_data;
 
-    /* The time when the thread is set to wake up. */
+    /*
+     * The time when the thread is set to wake up.  Depending on the
+     * situation we can be using timeval or timespec, so we need both.
+     */
     struct timeval wake_time;
-    volatile struct timespec wait_time;
+    union {
+#ifdef BROKEN_PSELECT
+	volatile struct timeval tv;
+#endif
+	volatile struct timespec ts;
+    } wait_time;
 #ifdef BROKEN_PSELECT
     bool signalled;
 #endif
@@ -214,9 +247,9 @@ struct selector_s
 
     int wake_sig;
 
-#ifdef HAVE_EPOLL_PWAIT
-    int epollfd;
-#endif
+    /* Used for epoll and kevent.  -1 otherwise. */
+    int evfd;
+
     sel_lock_t *(*sel_lock_alloc)(void *cb_data);
     void (*sel_lock_free)(sel_lock_t *);
     void (*sel_lock)(sel_lock_t *);
@@ -234,6 +267,20 @@ struct selector_s
 			   this code. */
 };
 
+static int *handle_set;
+static void (*handle_sig)(int sig, void *data);
+static void *handle_sig_data;
+
+void
+sel_set_handle_sig(int *ihandle_set,
+		   void (*ihandle_sig)(int sig, void *data),
+		   void *ihandle_sig_data)
+{
+    handle_set = ihandle_set;
+    handle_sig = ihandle_sig;
+    handle_sig_data = ihandle_sig_data;
+}
+
 static void
 sel_timer_lock(struct selector_s *sel)
 {
@@ -262,6 +309,38 @@ sel_fd_unlock(struct selector_s *sel)
 	sel->sel_unlock(sel->fd_lock);
 }
 
+#ifdef USE_PTHREADS
+#define sel_set_sigmask(newmask, oldmask)			\
+    pthread_sigmask(SIG_SETMASK, newmask, oldmask)
+#else
+#define sel_set_sigmask(newmask, oldmask)			\
+    (sigprocmask(SIG_SETMASK, newmask, oldmask) == -1 ? errno : 0)
+#endif
+
+#ifdef BROKEN_PSELECT
+static void pre_signal(sel_wait_list_t *item)
+{
+    item->signalled = true;
+    memset(&item->wait_time, 0, sizeof(item->wait_time));
+}
+static void pre_kill_one(struct selector_s *sel, long thread_id)
+{
+    sel_wait_list_t *item = NULL;
+
+    item = sel->wait_list.next;
+    while (item != &sel->wait_list) {
+	if (thread_id == item->thread_id) {
+	    pre_signal(item);
+	    break;
+	}
+	item = item->next;
+    }
+}
+#else
+#define pre_signal(item) do {} while(false)
+#define pre_kill_one(sel, thread_id) do {} while(false)
+#endif
+
 /* This function will wake the SEL thread.  It must be called with the
    timer lock held, because it messes with timeout.
 
@@ -285,11 +364,7 @@ i_wake_sel_thread(struct selector_s *sel, struct timeval *new_timeout)
 	if (item->send_sig && (!new_timeout ||
 			       cmp_timeval(new_timeout, &item->wake_time) < 0))
 	{
-#ifdef BROKEN_PSELECT
-	    item->signalled = true;
-	    item->wait_time.tv_sec = 0;
-	    item->wait_time.tv_nsec = 0;
-#endif
+	    pre_signal(item);
 	    item->send_sig(item->thread_id, item->send_sig_cb_data);
 	}
 	item = item->next;
@@ -311,11 +386,7 @@ i_sel_wake_first(struct selector_s *sel)
 
     item = sel->wait_list.next;
     if (item->send_sig && item != &sel->wait_list) {
-#ifdef BROKEN_PSELECT
-	item->signalled = true;
-	item->wait_time.tv_sec = 0;
-	item->wait_time.tv_nsec = 0;
-#endif
+	pre_signal(item);
 	item->send_sig(item->thread_id, item->send_sig_cb_data);
     }
 }
@@ -325,23 +396,9 @@ void
 sel_wake_one(struct selector_s *sel, long thread_id, sel_send_sig_cb killer,
 	     void *cb_data)
 {
-#ifdef BROKEN_PSELECT
-    sel_wait_list_t *item;
-#endif
 
     sel_timer_lock(sel);
-#ifdef BROKEN_PSELECT
-    item = sel->wait_list.next;
-    while (item != &sel->wait_list) {
-	if (thread_id == item->thread_id) {
-	    item->signalled = true;
-	    item->wait_time.tv_sec = 0;
-	    item->wait_time.tv_nsec = 0;
-	    break;
-	}
-	item = item->next;
-    }
-#endif
+    pre_kill_one(sel, thread_id);
     killer(thread_id, cb_data);
     sel_timer_unlock(sel);
 }
@@ -405,7 +462,7 @@ sel_update_fd(struct selector_s *sel, fd_control_t *fdc, int op)
     struct epoll_event event;
     int rv;
 
-    if (sel->epollfd < 0)
+    if (sel->evfd < 0)
 	return 1;
 
     memset(&event, 0, sizeof(event));
@@ -432,13 +489,74 @@ sel_update_fd(struct selector_s *sel, fd_control_t *fdc, int op)
     }
     /* This should only fail due to system problems, and if that's the case,
        well, we should probably terminate. */
-    rv = epoll_ctl(sel->epollfd, op, fdc->fd, &event);
+    rv = epoll_ctl(sel->evfd, op, fdc->fd, &event);
     if (rv) {
 	perror("epoll_ctl");
 	assert(0);
     }
     return 0;
 }
+#elif defined(USE_KEVENT)
+static int
+sel_update_fd(struct selector_s *sel, fd_control_t *fdc, int op)
+{
+    struct kevent chlist[3];
+    int rv;
+    struct timespec zerotime = { 0, 0 };
+
+    if (sel->evfd < 0)
+	return 1;
+
+    memset(chlist, 0, sizeof(chlist));
+
+    chlist[0].ident = fdc->fd;
+    if (op == SEL_FD_DEL)
+	chlist[0].flags |= EV_DELETE;
+    else
+	chlist[0].flags |= EV_ADD | EV_CLEAR;
+    chlist[1] = chlist[0];
+    chlist[2] = chlist[0];
+
+    chlist[0].filter = EVFILT_READ;
+    chlist[1].filter = EVFILT_WRITE;
+    chlist[2].filter = EVFILT_EXCEPT;
+
+    if (op != SEL_FD_DEL) {
+	if (fdc->read_enabled)
+	    chlist[0].flags |= EV_ENABLE;
+	else
+	    chlist[0].flags |= EV_DISABLE;
+	if (fdc->write_enabled)
+	    chlist[1].flags |= EV_ENABLE;
+	else
+	    chlist[1].flags |= EV_DISABLE;
+	if (fdc->except_enabled) {
+	    chlist[2].flags |= EV_ENABLE;
+	    chlist[2].fflags |= NOTE_OOB;
+	} else {
+	    chlist[2].flags |= EV_DISABLE;
+	}
+    }
+
+ retry:
+    if (fdc->iodir == SEL_RDWR) {
+	rv = kevent(sel->evfd, chlist, 3, NULL, 0, &zerotime);
+	if (rv == -1) {
+	    if (errno == EINVAL) {
+		/* Non-sockets may not accept EVFILT_EXCEPT, retry without. */
+		fdc->iodir = SEL_RDWR_NOEXC;
+		goto retry;
+	    }
+	}
+    } else {
+	rv = kevent(sel->evfd, chlist, 2, NULL, 0, &zerotime);
+    }
+    if (rv) {
+	perror("kevent");
+	assert(0);
+    }
+    return 0;
+}
 #else
 static int
 sel_update_fd(struct selector_s *sel, fd_control_t *fdc, int op)
@@ -494,10 +612,8 @@ sel_set_fd_handlers(struct selector_s *sel,
     void         *olddata = NULL;
     int          added = 1;
 
-#ifdef HAVE_EPOLL_PWAIT
-    if (sel->epollfd < 0 && fd >= FD_SETSIZE)
+    if (sel->evfd < 0 && fd >= FD_SETSIZE)
 	return EMFILE;
-#endif
 
     state = sel_alloc(sizeof(*state));
     if (!state)
@@ -542,10 +658,10 @@ sel_set_fd_handlers(struct selector_s *sel,
 	if (fd > sel->maxfd)
 	    sel->maxfd = fd;
 
-	if (sel_update_fd(sel, fdc, EPOLL_CTL_ADD))
+	if (sel_update_fd(sel, fdc, SEL_FD_ADD))
 	    sel_wake_all(sel);
     } else {
-	if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
+	if (sel_update_fd(sel, fdc, SEL_FD_MOD))
 	    sel_wake_all(sel);
     }
     sel_fd_unlock(sel);
@@ -576,7 +692,7 @@ i_sel_clear_fd_handler(struct selector_s *sel, int fd, int rpt)
 	olddata = fdc->data;
 	fdc->state = NULL;
 
-	sel_update_fd(sel, fdc, EPOLL_CTL_DEL);
+	sel_update_fd(sel, fdc, SEL_FD_DEL);
 #ifdef HAVE_EPOLL_PWAIT
 	fdc->saved_events = 0;
 #endif
@@ -584,10 +700,7 @@ i_sel_clear_fd_handler(struct selector_s *sel, int fd, int rpt)
     }
 
     init_fd(fdc);
-#ifdef HAVE_EPOLL_PWAIT
-    if (sel->epollfd < 0)
-#endif
-    {
+    if (sel->evfd < 0) {
 	FD_CLR(fd, (fd_set *) &sel->read_set);
 	FD_CLR(fd, (fd_set *) &sel->write_set);
 	FD_CLR(fd, (fd_set *) &sel->except_set);
@@ -647,20 +760,16 @@ sel_set_fd_read_handler(struct selector_s *sel, int fd, int state)
 	if (fdc->read_enabled)
 	    goto out;
 	fdc->read_enabled = 1;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_SET(fd, (fd_set *) &sel->read_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->read_enabled)
 	    goto out;
 	fdc->read_enabled = 0;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_CLR(fd, (fd_set *) &sel->read_set);
     }
-    if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
+    if (sel_update_fd(sel, fdc, SEL_FD_MOD))
 	sel_wake_all(sel);
 
  out:
@@ -684,20 +793,16 @@ sel_set_fd_write_handler(struct selector_s *sel, int fd, int state)
 	if (fdc->write_enabled)
 	    goto out;
 	fdc->write_enabled = 1;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_SET(fd, (fd_set *) &sel->write_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->write_enabled)
 	    goto out;
 	fdc->write_enabled = 0;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_CLR(fd, (fd_set *) &sel->write_set);
     }
-    if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
+    if (sel_update_fd(sel, fdc, SEL_FD_MOD))
 	sel_wake_all(sel);
 
  out:
@@ -721,20 +826,16 @@ sel_set_fd_except_handler(struct selector_s *sel, int fd, int state)
 	if (fdc->except_enabled)
 	    goto out;
 	fdc->except_enabled = 1;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_SET(fd, (fd_set *) &sel->except_set);
     } else if (state == SEL_FD_HANDLER_DISABLED) {
 	if (!fdc->except_enabled)
 	    goto out;
 	fdc->except_enabled = 0;
-#ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd < 0)
-#endif
+	if (sel->evfd < 0)
 	    FD_CLR(fd, (fd_set *) &sel->except_set);
     }
-    if (sel_update_fd(sel, fdc, EPOLL_CTL_MOD))
+    if (sel_update_fd(sel, fdc, SEL_FD_MOD))
 	sel_wake_all(sel);
 
  out:
@@ -1145,11 +1246,7 @@ setup_my_sigmask(sigset_t *sigmask, sigset_t *isigmask)
     if (isigmask) {
 	*sigmask = *isigmask;
     } else {
-#ifdef USE_PTHREADS
-	pthread_sigmask(SIG_SETMASK, NULL, sigmask);
-#else
-	sigprocmask(SIG_SETMASK, NULL, sigmask);
-#endif
+	sel_set_sigmask(NULL, sigmask);
     }
 }
 
@@ -1159,9 +1256,7 @@ setup_my_sigmask(sigset_t *sigmask, sigset_t *isigmask)
  * 	  <  0  when error
  */
 static int
-process_fds(struct selector_s *sel,
-	    sel_wait_list_t *item,
-	    sigset_t *isigmask)
+process_fds(struct selector_s *sel, sel_wait_list_t *item, sigset_t *isigmask)
 {
     fd_set      tmp_read_set;
     fd_set      tmp_write_set;
@@ -1170,12 +1265,13 @@ process_fds(struct selector_s *sel,
     int err;
     int num_fds;
     sigset_t sigmask;
-    unsigned long entry_fd_del_count = sel->fd_del_count;
+    unsigned long entry_fd_del_count;
     fd_control_t *fdc;
 
     setup_my_sigmask(&sigmask, isigmask);
- retry:
+
     sel_fd_lock(sel);
+    entry_fd_del_count = sel->fd_del_count;
     memcpy(&tmp_read_set, (void *) &sel->read_set, sizeof(tmp_read_set));
     memcpy(&tmp_write_set, (void *) &sel->write_set, sizeof(tmp_write_set));
     memcpy(&tmp_except_set, (void *) &sel->except_set, sizeof(tmp_except_set));
@@ -1183,15 +1279,68 @@ process_fds(struct selector_s *sel,
     sel_fd_unlock(sel);
 
     sigdelset(&sigmask, sel->wake_sig);
+
+#ifdef BROKEN_PSELECT
+    do {
+	int old_errno;
+	sigset_t oldmask;
+	bool time_fixed = false;
+
+	/*
+	 * A signal may have been sent before this process was queued on a
+	 * wait queue, so we may have the signal pending without having
+	 * been queued or had the time set to zero.  We detect that here
+	 * and set things up properly.
+	 */
+	sigpending(&oldmask);
+	if (!item->signalled && sigismember(&oldmask, sel->wake_sig)) {
+	    pre_signal(item);
+	    time_fixed = true;
+	}
+
+	err = sel_set_sigmask(&sigmask, &oldmask);
+	if (err < 0)
+	    return err;
+
+	/* If we don't have the wake sig blocked, fix it. */
+	if (!sigismember(&oldmask, sel->wake_sig)) {
+	    sigaddset(&oldmask, sel->wake_sig);
+	    /*
+	     * Make sure this returns immediately, just in case we
+	     * were signalled.
+	     */
+	    item->wait_time.tv.tv_sec = 0;
+	    item->wait_time.tv.tv_usec = 0;
+	    time_fixed = true;
+	}
+
+	err = select(num_fds,
+		     &tmp_read_set,
+		     &tmp_write_set,
+		     &tmp_except_set,
+		     (struct timeval *) &item->wait_time.tv);
+
+	if (time_fixed && err == 0) {
+	    err = -1;
+	    errno = EINTR;
+	}
+
+	old_errno = errno;
+	sel_set_sigmask(&oldmask, NULL);
+	errno = old_errno;
+    } while(false);
+#else
     err = pselect(num_fds,
 		  &tmp_read_set,
 		  &tmp_write_set,
 		  &tmp_except_set,
-		  (struct timespec *) &item->wait_time, &sigmask);
+		  (struct timespec *) &item->wait_time.ts, &sigmask);
+#endif
+
     if (err < 0) {
 	if (errno == EBADF || errno == EBADFD)
-	    /* We raced, just retry it. */
-	    goto retry;
+	    /* We raced, just retry it.  No loop here, timeout may be wrong. */
+	    errno = EINTR;
 	goto out;
     }
 
@@ -1226,7 +1375,7 @@ process_fds(struct selector_s *sel,
 
 #ifdef HAVE_EPOLL_PWAIT
 static int
-process_fds_epoll(struct selector_s *sel, struct timespec *tstimeout,
+process_fds_epoll(struct selector_s *sel, sel_wait_list_t *item,
 		  sigset_t *isigmask)
 {
     int rv;
@@ -1234,21 +1383,24 @@ process_fds_epoll(struct selector_s *sel, struct timespec *tstimeout,
     int timeout;
     sigset_t sigmask;
     fd_control_t *fdc;
-    unsigned long entry_fd_del_count = sel->fd_del_count;
+    unsigned long entry_fd_del_count;
 
     setup_my_sigmask(&sigmask, isigmask);
 
-    if (tstimeout->tv_sec > 600)
+    if (item->wait_time.ts.tv_sec > 600)
 	 /* Don't wait over 10 minutes, to work around an old epoll bug
 	    and avoid issues with timeout overflowing on 64-bit systems,
 	    which is much larger that 10 minutes, but who cares. */
 	timeout = 600 * 1000;
     else
-	timeout = ((tstimeout->tv_sec * 1000) +
-		   (tstimeout->tv_nsec + 999999) / 1000000);
+	timeout = ((item->wait_time.ts.tv_sec * 1000) +
+		   (item->wait_time.ts.tv_nsec + 999999) / 1000000);
 
     sigdelset(&sigmask, sel->wake_sig);
-    rv = epoll_pwait(sel->epollfd, &event, 1, timeout, &sigmask);
+    sel_fd_lock(sel);
+    entry_fd_del_count = sel->fd_del_count;
+    sel_fd_unlock(sel);
+    rv = epoll_pwait(sel->evfd, &event, 1, timeout, &sigmask);
     if (rv <= 0)
 	return rv;
 
@@ -1269,7 +1421,7 @@ process_fds_epoll(struct selector_s *sel, struct timespec *tstimeout,
 	 * EPOLLHUP or EPOLLERR, anyway, and then doing the callback
 	 * by hand.
 	 */
-	sel_update_fd(sel, fdc, EPOLL_CTL_DEL);
+	sel_update_fd(sel, fdc, SEL_FD_DEL);
 	fdc->saved_events = event.events & (EPOLLHUP | EPOLLERR);
 	/*
 	 * Have it handle read data, too, so if there is a pending
@@ -1290,7 +1442,7 @@ process_fds_epoll(struct selector_s *sel, struct timespec *tstimeout,
  rearm:
     /* Rearm the event.  Remember it could have been deleted in the handler. */
     if (fdc->state)
-	sel_update_fd(sel, fdc, EPOLL_CTL_MOD);
+	sel_update_fd(sel, fdc, SEL_FD_MOD);
     sel_fd_unlock(sel);
 
     return rv;
@@ -1308,16 +1460,161 @@ sel_setup_forked_process(struct selector_s *sel)
      * be independent.  If you don't do this, disabling an fd in the
      * child disables the parent, too, and vice versa.
      */
-    close(sel->epollfd);
-    sel->epollfd = epoll_create(32768);
-    if (sel->epollfd == -1) {
+    close(sel->evfd);
+    sel->evfd = epoll_create(32768);
+    if (sel->evfd == -1) {
 	return errno;
     }
 
     for (i = 0; i <= sel->maxfd; i++) {
 	fd_control_t *fdc = sel->fds[i];
 	if (fdc && fdc->state)
-	    sel_update_fd(sel, fdc, EPOLL_CTL_ADD);
+	    sel_update_fd(sel, fdc, SEL_FD_ADD);
+    }
+    return 0;
+}
+#elif defined(USE_KEVENT)
+static int
+process_fds_kevent(struct selector_s *sel, sel_wait_list_t *item,
+		   sigset_t *isigmask)
+{
+    int rv, old_errno;
+    struct kevent event;
+#define MAX_KEVENT_SIGS 10
+    struct kevent sigs[MAX_KEVENT_SIGS];
+    sigset_t sigmask, oldmask, pendmask;
+    fd_control_t *fdc;
+    unsigned long entry_fd_del_count;
+    unsigned int i, j;
+    bool time_fixed = false;
+
+    setup_my_sigmask(&sigmask, isigmask);
+    sigdelset(&sigmask, sel->wake_sig);
+
+    sel_fd_lock(sel);
+    entry_fd_del_count = sel->fd_del_count;
+    sel_fd_unlock(sel);
+
+    /*
+     * kevent doesn't have a pwait version.  From what I can tell,
+     * BSDs all have broken pselect.  See comments in process_fds()
+     * about this.
+     */
+    rv = sel_set_sigmask(NULL, &oldmask);
+    if (rv < 0)
+	return rv;
+
+    sigpending(&pendmask);
+    if (!item->signalled && sigismember(&pendmask, sel->wake_sig)) {
+	pre_signal(item);
+	time_fixed = true;
+    }
+
+    /*
+     * If a signal is blocked in the running thread but would be
+     * unblocked by the setmask, leave it blocked for the thread and
+     * return it from kevent.
+     */
+    for (i = 0, j = 0; handle_set && handle_set[i]; i++) {
+	assert(j < MAX_KEVENT_SIGS);
+	if (!sigismember(&oldmask, handle_set[i])) {
+	    sigaddset(&oldmask, handle_set[i]);
+	    item->wait_time.tv.tv_sec = 0;
+	    item->wait_time.tv.tv_usec = 0;
+	    time_fixed = true;
+	}
+	if (!sigismember(&sigmask, handle_set[i])) {
+	    EV_SET(&sigs[j], handle_set[i],
+		   EVFILT_SIGNAL, EV_ADD | EV_ENABLE, 0, 0, 0);
+	    j++;
+	    sigaddset(&sigmask, handle_set[i]);
+	}
+    }
+
+    rv = sel_set_sigmask(&sigmask, NULL);
+    if (rv < 0)
+	return rv;
+
+    /* If we don't have the wake sig blocked, fix it. */
+    if (!sigismember(&oldmask, sel->wake_sig)) {
+	sigaddset(&oldmask, sel->wake_sig);
+	/*
+	 * Make sure this returns immediately, just in case we
+	 * were signalled.
+	 */
+	item->wait_time.tv.tv_sec = 0;
+	item->wait_time.tv.tv_usec = 0;
+	time_fixed = true;
+    }
+
+    rv = kevent(sel->evfd, sigs, j, &event, 1,
+		(struct timespec *) &item->wait_time.ts);
+
+    if (time_fixed && rv == 0) {
+	rv = -1;
+	errno = EINTR;
+    }
+
+    old_errno = errno;
+    sel_set_sigmask(&oldmask, NULL);
+    errno = old_errno;
+
+    if (rv <= 0)
+	return rv;
+
+    if (event.filter == EVFILT_SIGNAL) {
+	handle_sig(event.ident, handle_sig_data);
+	errno = EINTR;
+	return -1;
+    }
+
+    sel_fd_lock(sel);
+    valid_fd(sel, event.ident, &fdc);
+    if (entry_fd_del_count != sel->fd_del_count)
+	/* Something was deleted from the FD set, don't process this as it
+	   may be from the old fd wakeup. */
+	goto rearm;
+    if (event.filter == EVFILT_READ) {
+	handle_selector_call(sel, fdc, NULL, fdc->read_enabled,
+			     fdc->handle_read);
+    } else if (event.filter == EVFILT_WRITE) {
+	handle_selector_call(sel, fdc, NULL, fdc->write_enabled,
+			     fdc->handle_write);
+    } else if (event.filter == EVFILT_EXCEPT) {
+	handle_selector_call(sel, fdc, NULL, fdc->except_enabled,
+			     fdc->handle_except);
+    }
+
+ rearm:
+    /* Rearm the event.  Remember it could have been deleted in the handler. */
+    if (fdc->state)
+	sel_update_fd(sel, fdc, SEL_FD_MOD);
+    sel_fd_unlock(sel);
+
+    return rv;
+}
+
+int
+sel_setup_forked_process(struct selector_s *sel)
+{
+    int i;
+
+    /*
+     * More epoll stupidity.  In a forked process we must create a new
+     * epoll because the epoll state is shared between a parent and a
+     * child.  If it worked like it should, each epoll instance would
+     * be independent.  If you don't do this, disabling an fd in the
+     * child disables the parent, too, and vice versa.
+     */
+    close(sel->evfd);
+    sel->evfd = kqueue();
+    if (sel->evfd == -1) {
+	return errno;
+    }
+    for (i = 0; i <= sel->maxfd; i++) {
+	fd_control_t *fdc = sel->fds[i];
+	if (fdc && fdc->state)
+	    sel_update_fd(sel, fdc, SEL_FD_ADD);
     }
     return 0;
 }
@@ -1365,18 +1662,30 @@ sel_select_intr_sigmask(struct selector_s *sel,
 
 	memset(&wait_entry, 0, sizeof(wait_entry));
 
-	wait_entry.wait_time.tv_sec = tmp_timeout.tv_sec;
-	wait_entry.wait_time.tv_nsec = tmp_timeout.tv_usec * 1000;
+#ifdef BROKEN_PSELECT
+	if (sel->evfd >= 0) {
+	    wait_entry.wait_time.ts.tv_sec = tmp_timeout.tv_sec;
+	    wait_entry.wait_time.ts.tv_nsec = tmp_timeout.tv_usec * 1000;
+	} else {
+	    wait_entry.wait_time.tv = tmp_timeout;
+	}
+#else
+	wait_entry.wait_time.ts.tv_sec = tmp_timeout.tv_sec;
+	wait_entry.wait_time.ts.tv_nsec = tmp_timeout.tv_usec * 1000;
+#endif
 
 	add_sel_wait_list(sel, &wait_entry, send_sig, cb_data, thread_id,
 			  &wake_time);
 	sel_timer_unlock(sel);
 
 #ifdef HAVE_EPOLL_PWAIT
-	if (sel->epollfd >= 0)
-	    err = process_fds_epoll(sel,
-				    (struct timespec *) &wait_entry.wait_time,
-				    sigmask);
+	if (sel->evfd >= 0)
+	    err = process_fds_epoll(sel, &wait_entry, sigmask);
+	else
+#endif
+#ifdef USE_KEVENT
+	if (sel->evfd >= 0)
+	    err = process_fds_kevent(sel, &wait_entry, sigmask);
 	else
 #endif
 	    err = process_fds(sel, &wait_entry, sigmask);
@@ -1531,13 +1840,7 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 
     sigemptyset(&sigset);
     sigaddset(&sigset, wake_sig);
-#ifdef USE_PTHREADS
-    rv = pthread_sigmask(SIG_BLOCK, &sigset, NULL);
-#else
-    rv = sigprocmask(SIG_BLOCK, &sigset, NULL);
-    if (rv == -1)
-	rv = errno;
-#endif
+    rv = sel_set_sigmask(&sigset, NULL);
     if (rv) {
 	if (sel->sel_lock_alloc) {
 	    sel->sel_lock_free(sel->fd_lock);
@@ -1547,11 +1850,17 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 	return rv;
     }
 
+    sel->evfd = -1;
 #ifdef HAVE_EPOLL_PWAIT
-    sel->epollfd = epoll_create(32768);
-    if (sel->epollfd == -1)
+    sel->evfd = epoll_create(32768);
+    if (sel->evfd == -1)
 	syslog(LOG_ERR, "Unable to set up epoll, falling back to select: %m");
 #endif
+#ifdef USE_KEVENT
+    sel->evfd = kqueue();
+    if (sel->evfd == -1)
+	syslog(LOG_ERR, "Unable to set up kevent, falling back to select: %m");
+#endif
 
     *new_selector = sel;
 
@@ -1578,10 +1887,8 @@ sel_free_selector(struct selector_s *sel)
 	free(elem);
 	elem = theap_get_top(&(sel->timer_heap));
     }
-#ifdef HAVE_EPOLL_PWAIT
-    if (sel->epollfd >= 0)
-	close(sel->epollfd);
-#endif
+    if (sel->evfd >= 0)
+	close(sel->evfd);
     for (i = 0; i < FD_SETSIZE; i++) {
 	while (sel->fds[i]) {
 	    fd_control_t *fdc = sel->fds[i];
