@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <signal.h>
+#include <time.h>
 #include <gensio/gensio.h>
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_osops_env.h>
@@ -761,7 +762,7 @@ struct oom_tests oom_tests[] = {
     { "udp,ipv4,localhost,", "udp,ipv4,0",
       /* In this tests some errors will not result in a failure. */
       .allow_no_err_on_trig = true,
-      .max_io_size = 2000
+      .max_io_size = 1000
     },
     { "mux,sctp,localhost,", "mux,sctp,0",
       .check_if_present = check_sctp_present, .check_value = HAVE_LIBSCTP,
@@ -1159,6 +1160,7 @@ con_cb(struct gensio *io, void *user_data,
 	    gensio_set_write_callback_enable(io, false);
 	    gensio_set_read_callback_enable(io, false);
 	    id->err = OOME_DATA_MISMATCH;
+	    assert(0);
 	    gensio_os_funcs_wake(o, od->waiter);
 	}
 
@@ -2182,6 +2184,7 @@ main(int argc, char *argv[])
     unsigned long errcount = 0, skipcount = 0, testcount = 0;
     unsigned int repeat_count = 1;
     int testnr = -1, numtests = 0, testnrstart = -1, testnrend = MAX_LOOPS;
+    int num_oom_threads = 3;
     gensio_time zerotime = { 0, 0 };
     struct oom_tests user_test;
     bool list_tests = false;
@@ -2190,6 +2193,10 @@ main(int argc, char *argv[])
     char *s;
     struct env_info env;
 
+#if !defined(HAVE_GETRANDOM_FUNC)
+    srand(time(NULL));
+#endif
+
     memset(&user_test, 0, sizeof(user_test));
 
     if (fill_random(&iodata_size, sizeof(iodata_size)))
@@ -2281,6 +2288,13 @@ main(int argc, char *argv[])
 		exit(1);
 	    }
 	    num_extra_threads = strtol(argv[i], NULL, 0);
+	} else if (strcmp(argv[i], "-o") == 0) {
+	    i++;
+	    if (i >= argc) {
+		fprintf(stderr, "No number given with -o\n");
+		exit(1);
+	    }
+	    num_oom_threads = strtol(argv[i], NULL, 0);
 	} else if (strcmp(argv[i], "-e") == 0) {
 	    i++;
 	    if (i >= argc) {
@@ -2334,10 +2348,12 @@ main(int argc, char *argv[])
 	fprintf(stderr, "tcl specified, but tcl OS handler not available.\n");
 	exit(1);
 #else
-	if (num_extra_threads > 0)
-	    fprintf(stderr, "Number of extra threads is %u, incompatible with"
-		    " TCL, forcing to 0\n", num_extra_threads);
+	if (num_extra_threads > 0 || num_oom_threads > 0)
+	    fprintf(stderr, "Number of extra threads is (%u,%u),"
+		    " incompatible with TCL, forcing to 0\n",
+		    num_extra_threads, num_oom_threads);
 	num_extra_threads = 0;
+	num_oom_threads = 0;
 	os_func_str = " --tcl";
 	rv = gensio_tcl_funcs_alloc(&o);
 #endif
@@ -2397,7 +2413,7 @@ main(int argc, char *argv[])
     if (i >= argc)
 	gensio_os_funcs_zfree(o, s);
 
-    for (i = 0; i < num_extra_threads; i++) {
+    for (i = 0; i < num_oom_threads; i++) {
 	loopwaiter[i] = gensio_os_funcs_alloc_waiter(o);
 	if (!loopwaiter[i]) {
 	    fprintf(stderr, "Could not allocate loop waiter\n");
@@ -2440,7 +2456,7 @@ main(int argc, char *argv[])
 
     gensio_argv_free(o, env.argv);
 
-    for (i = 0; i < num_extra_threads; i++) {
+    for (i = 0; i < num_oom_threads; i++) {
 	gensio_os_funcs_wake(o, loopwaiter[i]);
 	gensio_os_wait_thread(loopth[i]);
 	gensio_os_funcs_free_waiter(o, loopwaiter[i]);
