@@ -10,7 +10,7 @@ import gensio
 
 check_sctp()
 print("Test sctp accepter connect")
-TestAcceptConnect(o, "sctp,0", "sctp,0", "sctp,localhost,",
+TestAcceptConnect(o, "sctp,localhost,0", "sctp,localhost,0", "sctp,localhost,",
                   do_small_test)
 del o
 test_shutdown()
