@@ -9,8 +9,7 @@ from utils import *
 import gensio
 
 print("Test telnet small")
-TestAccept(o, "telnet,tcp,localhost,", "telnet(rfc2217=true),tcp,0",
-           do_small_test, chunksize = 64, is_sergensio = True,
-           enable_read_io1 = True)
+TestAccept(o, "telnet,tcp,localhost,", "telnet(rfc2217=true),tcp,localhost,0",
+           do_small_test, chunksize = 64, enable_read_io1 = True)
 del o
 test_shutdown()
