@@ -32,6 +32,7 @@
 #include <gensio/gensio_mdns.h>
 #include <gensio/gensio_selector.h>
 #include <gensio/argvutils.h>
+#include <gensio/gensio_time.h>
 #include "utils.h"
 
 static const char *progname;
@@ -86,10 +87,20 @@ ip_domain_to_str(int ipdomain)
     }
 }
 
+static struct gensio_timer *close_timer;
+
+static void
+check_close_done(struct gensio_timer *t, void *cb_data)
+{
+    struct freed_data *f = cb_data;
+
+    gensio_os_funcs_wake(f->o, f->closewaiter);
+}
+
 static void
 mdns_info_found(struct gensio_mdns_watch *w,
 		enum gensio_mdns_data_state state,
-		int interface, int ipdomain,
+		int ifinterface, int ipdomain,
 		const char *name, const char *type,
 		const char *domain, const char *host,
 		const struct gensio_addr *addr,
@@ -100,16 +111,19 @@ mdns_info_found(struct gensio_mdns_watch *w,
     int rv;
     struct freed_data *f = userdata;
 
-    if (state == GENSIO_MDNS_ALL_FOR_NOW) {
+    if (state == GENSIO_MDNS_WATCH_ALL_FOR_NOW) {
 	printf("All-for-now:\n");
-	if (close_on_done)
-	    gensio_os_funcs_wake(f->o, f->closewaiter);
+	if (close_on_done) {
+	    struct gensio_time timeout = {1, 0};
+	    gensio_os_funcs_stop_timer(f->o, close_timer);
+	    gensio_os_funcs_start_timer(f->o, close_timer, &timeout);
+	}
 	return;
     }
 
     printf("%s:\n interface: %d\n iptype: %s\n",
-	   state == GENSIO_MDNS_NEW_DATA ? "Found" : "Removed",
-	   interface, ip_domain_to_str(ipdomain));
+	   state == GENSIO_MDNS_WATCH_NEW_DATA ? "Found" : "Removed",
+	   ifinterface, ip_domain_to_str(ipdomain));
     printf(" name: '%s'\n type: '%s'\n domain: '%s'\n host: '%s'\n",
 	   name, type, domain, host);
     rv = gensio_addr_to_str(addr, strbuf, NULL, sizeof(strbuf));
@@ -159,6 +173,35 @@ help(int err)
     gensio_osfunc_exit(err);
 }
 
+static void
+mdns_service_cb(struct gensio_mdns_service *service,
+		enum gensio_mdns_service_event ev,
+		const char *info,
+		void *cb_data)
+{
+    switch(ev) {
+    case GENSIO_MDNS_SERVICE_ERROR:
+	printf("Service registration error: %s\n", info);
+	break;
+
+    case GENSIO_MDNS_SERVICE_READY:
+	printf("Service is ready: %s\n", info);
+	break;
+
+    case GENSIO_MDNS_SERVICE_READY_NEW_NAME:
+	printf("Service is ready and has a new name: %s\n", info);
+	break;
+
+    case GENSIO_MDNS_SERVICE_REMOVED: {
+	struct freed_data *f = cb_data;
+
+	gensio_os_funcs_wake(f->o, f->closewaiter);
+	printf("Service is removed\n");
+	break;
+    }
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -170,17 +213,19 @@ main(int argc, char *argv[])
     struct freed_data fdata;
     int rv, arg, err;
     const char *name = NULL, *type = NULL, *domain = NULL, *host = NULL;
-    int interface = -1, nettype = GENSIO_NETTYPE_UNSPEC, port = -1;
+    int ifinterface = -1, nettype = GENSIO_NETTYPE_UNSPEC, port = -1;
     const char *nettype_str = NULL;
     const char *txtstr = NULL;
     const char **txt = NULL;
     gensiods txtargc = 0, txtargs = 0;
     bool do_service = false;
     struct gensio_os_proc_data *proc_data;
+    gensio_time timeout, *dotimeout = NULL;
+    unsigned int timeoutv;
 
     progname = argv[0];
 
-    rv = gensio_default_os_hnd(0, &o);
+    rv = gensio_alloc_os_funcs(0, &o, 0);
     if (rv) {
 	fprintf(stderr, "Could not allocate OS handler: %s\n",
 		gensio_err_to_str(rv));
@@ -200,8 +245,13 @@ main(int argc, char *argv[])
 	} else if ((rv = cmparg(argc, argv, &arg, "-o", "--host", &host))) {
 	    ;
 	} else if ((rv = cmparg_int(argc, argv, &arg, "-i", "--interface",
-				    &interface))) {
+				    &ifinterface))) {
 	    ;
+	} else if ((rv = cmparg_uint(argc, argv, &arg, NULL, "--timeout",
+				     &timeoutv))) {
+	    timeout.secs = timeoutv / 1000;
+	    timeout.nsecs = GENSIO_MSECS_TO_NSECS(timeoutv % 1000);
+	    dotimeout = &timeout;
 	} else if ((rv = cmparg(argc, argv, &arg, "-y", "--nettype",
 				&nettype_str))) {
 	    if (strcmp(nettype_str, "ipv4") == 0)
@@ -254,6 +304,15 @@ main(int argc, char *argv[])
 	return 1;
     }
 
+    if (close_on_done) {
+	close_timer = gensio_os_funcs_alloc_timer(o, check_close_done,
+						  &fdata);
+	if (!close_timer) {
+	    fprintf(stderr, "Could allocate close timer\n");
+	    goto out_err;
+	}
+    }
+
     closewaiter = gensio_os_funcs_alloc_waiter(o);
     if (!closewaiter) {
 	rv = GE_NOMEM;
@@ -298,26 +357,32 @@ main(int argc, char *argv[])
 			gensio_err_to_str(err));
 	    exit(1);
 	}
-	rv = gensio_mdns_add_service(mdns, interface, nettype,
-				     name, type, domain, host,
-				     port, txt, &service);
+	rv = gensio_mdns_add_service2(mdns, ifinterface, nettype,
+				      name, type, domain, host,
+				      port, txt,
+				      mdns_service_cb, &fdata,
+				      &service);
 	if (rv) {
 	    fprintf(stderr, "Could not allocate mdns service: %s\n",
 		    gensio_err_to_str(rv));
 	    goto out_err;
 	}
     } else {
-	rv = gensio_mdns_add_watch(mdns, interface, nettype,
+	rv = gensio_mdns_add_watch(mdns, ifinterface, nettype,
 				   name, type, domain, host,
 				   mdns_info_found, &fdata, &watch);
-	if (rv) {
+	if (rv == GE_INCONSISTENT) {
+	    /* This means we have to supply a type. */
+	    fprintf(stderr, "You must supply a type parameter to watch for.\n");
+	    goto out_err;
+	} else if (rv) {
 	    fprintf(stderr, "Could not allocate mdns watcher: %s\n",
 		    gensio_err_to_str(rv));
 	    goto out_err;
 	}
     }
 
-    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
+    gensio_os_funcs_wait(o, closewaiter, 1, dotimeout);
 
  out_err:
     if (service) {
@@ -325,6 +390,8 @@ main(int argc, char *argv[])
 	if (rv)
 	    fprintf(stderr, "Could not free mdns service: %s\n",
 		    gensio_err_to_str(rv));
+	else
+	    gensio_os_funcs_wait(o, closewaiter, 1, NULL);
     }
     if (watch) {
 	rv = gensio_mdns_remove_watch(watch, mdns_watch_freed, &fdata);
@@ -349,6 +416,10 @@ main(int argc, char *argv[])
 
     if (o && closewaiter)
 	gensio_os_funcs_free_waiter(o, closewaiter);
+
+    if (close_timer)
+	gensio_os_funcs_free_timer(o, close_timer);
+
     if (o) {
 	gensio_time endwait = { 0, 0 };
 
