@@ -1125,7 +1125,7 @@ main(int argc, char *argv[])
     bool use_tcl = false;
     gensio_time endwait = { 5, 0 };
     struct gensio *io = NULL;
-    bool noregterm; /* Don't register the termination handler. */
+    bool regterm = true; /* Register the termination handler. */
 
     memset(&g, 0, sizeof(g));
     g.escape_char = -1;
@@ -1189,7 +1189,7 @@ main(int argc, char *argv[])
 	} else if ((rv = cmparg(argc, argv, &arg, "-h", "--help", NULL)))
 	    help(0);
 	else if ((rv = cmparg(argc, argv, &arg, NULL, "--noregterm", NULL))) {
-	    noregterm = true;
+	    regterm = false;
 	} else if ((rv = cmparg(argc, argv, &arg, NULL, "--dummyrand",
 			      &filename))) {
 	    /*
@@ -1273,6 +1273,13 @@ main(int argc, char *argv[])
 	goto out_err;
     }
 
+    if (regterm) {
+	rv = gensio_os_proc_register_term_handler(proc_data, handle_term, &g);
+	if (rv)
+	    handle_term(&g);
+    }
+    gensio_os_proc_register_reload_handler(proc_data, handle_reload, &g);
+
     rv = alloc_threads(g.o);
     if (rv)
 	goto out_err;
@@ -1317,13 +1324,6 @@ main(int argc, char *argv[])
 	io = NULL;
     }
 
-    if (!noregterm) {
-	rv = gensio_os_proc_register_term_handler(proc_data, handle_term, &g);
-	if (rv)
-	    handle_term(&g);
-    }
-    gensio_os_proc_register_reload_handler(proc_data, handle_reload, &g);
-
     gensio_os_funcs_wait(g.o, g.waiter, 1, NULL);
 
  out_err:
